{*******************************************************}
{                                                       }
{    Copyright  1995-2002 by Lucian Radulescu          }
{    mailto  :  lucian@ez-delphi.com                    }
{    http    :  http://www.ez-delphi.com                }
{                                                       }
{*******************************************************}

{$I ALLLIB.DEF}

unit MonReg;

interface

uses
  Windows, SysUtils, Classes, Controls, Graphics, Forms, StdCtrls,
  {$IFDEF ATLEAST_D6} DesignIntf, DesignEditors,
  {$ELSE}             DsgnIntf,
  {$ENDIF ATLEAST_D6}
  Spin, Dialogs, ExtCtrls, PerfMon;

type

  // TLinesEditor
  TLinesEditor = class(TForm)
    CancelButton: TButton;
    OKButton: TButton;
    grpLines: TGroupBox;
    grpProperties: TGroupBox;
    btnAdd: TButton;
    btnDelete: TButton;
    ListBox1: TListBox;
    Label1: TLabel;
    Label3: TLabel;
    Edit1: TEdit;
    ColorDialog1: TColorDialog;
    Button1: TButton;
    SpinEdit1: TSpinEdit;
    Shape1: TShape;
    Label2: TLabel;
    Label4: TLabel;
    SpinEdit2: TSpinEdit;
    SpinEdit3: TSpinEdit;
    procedure Button1Click(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure RefreshItem(Sender: TObject);
    procedure Edit1Exit(Sender: TObject);
  private
    { Private declarations }
    FLines: TLineItems;
    Updating, Modified: Boolean;
    procedure SetLines(Value: TLineItems);
    procedure RefreshList;
  public
    { Public declarations }
    property EditorLines: TLineItems read FLines write SetLines;
  end;


  // TLineItemsProperty
  TLineItemsProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;


  // TLineItemsEditor
  TLineItemsEditor = class(TDefaultEditor)
  protected
    {$IFDEF ATLEAST_D6}
    procedure EditProperty(const Prop: IProperty; var Continue: Boolean); override;
    {$ELSE}
    procedure EditProperty(PropertyEditor: TPropertyEditor;
      var Continue , FreeEditor: Boolean); override;
    {$ENDIF ATLEAST_D6}
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;


procedure Register;


implementation


{$R *.DFM}
{$R PERFMON.DCR}

procedure Register;
begin
  RegisterComponents('JeeLIB', [TPerformanceGraphic]);
  RegisterComponentEditor(TPerformanceGraphic, TLineItemsEditor);
  RegisterPropertyEditor(TypeInfo(TLineItems), TPerformanceGraphic, 'Items', TLineItemsProperty);
end;


// TLineItemsProperty ----------------------------------------------------------

procedure TLineItemsProperty.Edit;
var
  LinesEditor: TLinesEditor;
begin
  LinesEditor := TLinesEditor.Create(Application);
  try
    LinesEditor.EditorLines := TLineItems(GetOrdValue);
    LinesEditor.ShowModal;
    if LinesEditor.Modified then
      Modified;
  finally
    LinesEditor.Free;
  end;
end;

function TLineItemsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;


// TLineItemsEditor ------------------------------------------------------------

procedure TLineItemsEditor.EditProperty;
var
  PropName: string;
begin
  {$IFDEF ATLEAST_D6} PropName := Prop.GetName;
  {$ELSE}             PropName := PropertyEditor.GetName;
  {$ENDIF ATLEAST_D6}
  if (CompareText(PropName, 'ITEMS') = 0) then
  begin
    {$IFDEF ATLEAST_D6} Prop.Edit;
    {$ELSE}             PropertyEditor.Edit;
    {$ENDIF ATLEAST_D6}
    Continue := False;
    {$IFNDEF ATLEAST_D6}FreeEditor := True;
    {$ENDIF ATLEAST_D6}
  end;
end;

function TLineItemsEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TLineItemsEditor.GetVerb(Index: Integer): string;
begin
  if Index = 0 then
    Result := 'Edit Lines'
  else
    Result := '';
end;

procedure TLineItemsEditor.ExecuteVerb(Index: Integer);
begin
  if Index = 0 then Edit;
end;

// TLinesEditor ----------------------------------------------------------------

procedure TLinesEditor.SetLines(Value: TLineItems);
begin
  Modified := False;
  FLines := Value;
  RefreshList;
end;

procedure TLinesEditor.RefreshList;
var
  I, OldIndex: Integer;
begin
  OldIndex := ListBox1.ItemIndex;
  ListBox1.Items.Clear;
  for I := 0 to FLines.Count - 1 do
    ListBox1.Items.Add(FLines.Item[I].Description);
  if (OldIndex = -1) and (FLines.Count > 0) then
    OldIndex := 0;
  ListBox1.ItemIndex := OldIndex;
  ListBox1Click(ListBox1);
end;

procedure TLinesEditor.Button1Click(Sender: TObject);
begin
  if ColorDialog1.Execute then
  begin
    RefreshItem(Sender);
    Shape1.Brush.Color := ColorDialog1.Color;
  end;
end;

procedure TLinesEditor.ListBox1Click(Sender: TObject);
begin
  btnDelete.Enabled := ListBox1.ItemIndex <> -1;
  if not btnDelete.Enabled then
    Exit;
  Updating := True;
  try
    with FLines.Item[ListBox1.ItemIndex] do
    begin
      Edit1.Text := Description;
      ColorDialog1.Color := Color;
      Shape1.Brush.Color := Color;
      SpinEdit1.Value := PenWidth;
      SpinEdit2.Value := MinValue;
      SpinEdit3.Value := MaxValue;
    end;
  finally
    Updating := False;
  end;
end;

procedure TLinesEditor.RefreshItem(Sender: TObject);
begin
  if Updating or (ListBox1.ItemIndex = -1) then
    Exit;

  with FLines.Item[ListBox1.ItemIndex] do
  begin
    Description := Edit1.Text;
    Color := ColorDialog1.Color;
    PenWidth := SpinEdit1.Value;
    MinValue := SpinEdit2.Value;
    MaxValue := SpinEdit3.Value;
  end;

  if Sender = Edit1 then
    RefreshList;
end;

procedure TLinesEditor.Edit1Change(Sender: TObject);
begin
  if Updating then
    Exit;
  btnAdd.Enabled := Edit1.Text <> '';
end;

procedure TLinesEditor.Edit1Exit(Sender: TObject);
begin
  RefreshItem(nil);
  RefreshList;
  Modified := True;
end;

procedure TLinesEditor.btnAddClick(Sender: TObject);
var
  aLine: TLineItem;
begin
  aLine := FLines.Add;
  ListBox1.ItemIndex := ListBox1.Items.Add(Format('Line %d', [ListBox1.Items.Count+1]));
  RefreshItem(Sender);
  Modified := True;
end;

procedure TLinesEditor.btnDeleteClick(Sender: TObject);
var
  OldIndex: Integer;
begin
  OldIndex := ListBox1.ItemIndex;
  FLines.Delete(ListBox1.ItemIndex);
  ListBox1.Items.Delete(ListBox1.ItemIndex);
  if OldIndex = ListBox1.Items.Count then
    Dec(OldIndex);
  ListBox1.ItemIndex := OldIndex;
  ListBox1Click(ListBox1);
  Modified := True;
end;

end.

